// Automdaq device protocol V1.000
// Serial link version
// Protocole Automdaq device V1.000

// IP version
// Version IP


byte analogPin = 0;		// first analog input pin, première entrée analogique

unsigned lenin; 
unsigned posframin;
unsigned char framin[256];
unsigned char crc;
unsigned phase=0;
int framereceived=0;
int run=0;
unsigned nchannels=1;
float scanpersec;
unsigned long mintimebetween2scansmicrosec;
unsigned long lastscantime;
int firstscan;
unsigned long lastcomtime;


void setup(void) 
{
run=0;
phase=0;
Serial.begin(115200); 
}


void processframein(void) // Process an incoming frame arriving from PC, traite une trame arrivant du PC
{
unsigned long curtime=millis();
if (Serial.available() == 0) 
  {
  if(run&&curtime-lastcomtime>11000) 
    {
    run=0; // Back to stop mode if nothing received during 11 secs, reroune en mode stop si pas de trame reçu du PC depuis 11 secondes
    }
  if(phase>0&&curtime-lastcomtime>1000) phase=0; // Timeout: back to phase 0, timeout : retour à la phase 0
  return;
  }
lastcomtime=curtime;
int charin=Serial.read();
crc+=charin;
switch(phase)
  {
  case 0 : // waiting for header char, attente du caractère d'entête
    {
    if(charin=='A') phase++; 
    crc='A';
    } break;
  case 1 : // waiting for frame len, attente de la longueur de la trame
    {
    lenin=charin; 
    if(lenin<5) phase=0;
    lenin-=3;
    posframin=0; 
    phase++; 
    } break;
  case 2 : // waiting for frame chars, attente des caractères de la trame
    {
     framin[posframin++]=charin;
     if(posframin==lenin) phase++;
    } break;
  case 3 : // waiting for crc, attente du crc
    {
    crc-=charin;
    crc=1-crc;
    if(crc==charin)
      {
      framereceived=1;
      }
    phase=0;
    } break;
  }
}

unsigned char buffout[256];

// send a frame (len max is 255-3), envoie une trame (longueur max 255-3)
// add header char len and crc at the end and then send, ajoute le caractère d'entête la longueur et le crc à la fin puis emmet
void sendframe(unsigned char *frame,unsigned char len)
{
len+=3;
buffout[0]='A';
buffout[1]=len;
memcpy(&buffout[2],frame,len-3);
unsigned char crc=0;
unsigned count;
for(count=0;count<len-1;count++)
  {
  crc+=buffout[count];
  }
buffout[count++]=1-crc;
Serial.write(buffout,len);
}

// process an incoming command (in a received frame), traite une commande entrante (dans une trame reçue)
void processcmd(void)
{
switch(framin[0])
  {
  case 1 : // Setup, configuration
    {
    if(run)
      {
      sendframe((unsigned char *)"\x81\xFF\x01",3); // not possible while running, impossible en mode run
      }
    else
      {
      if(framin[1]<1||framin[1]>4)
        {
        sendframe((unsigned char *)"\x81\xFF\x02",3); // bad channel number, numéro de voie invalide
        }
      else
        {
        scanpersec=-1; // no frequncy limitation by default, pas de limitation dans la fréquence d'échantillonage par défaut
        if(lenin>=6)
          {
          scanpersec=*((float *)&framin[2]);
          }
        if(scanpersec<=0) mintimebetween2scansmicrosec=0;
        else mintimebetween2scansmicrosec=(unsigned long)((float)1000000/scanpersec); 
        nchannels=framin[1];
        unsigned char frameout[11];
        frameout[0]=0x81; // answering setup request, réponse à la demande de configuration
        frameout[1]=0; // Ok
        frameout[2]=1; // 16 bits signed int datas, données signés 16 bits
        float div=1023; // values from 0 to 1023 for arduino analog inputs, valeur entre 0 et 1023 pour les entrées analogiques d'arduino
        float mul=5; // 0 to 5v, entre 0 et 5v
        memcpy(&frameout[3],&div,4);
        memcpy(&frameout[7],&mul,4);
        sendframe(frameout,11); 
        }
      }
    } break;
  case 2 : // Set mode, défini le mode
    {
    if(framin[1]==1) // Ask for run, demande le mode run
      {
      if(run==1)
        {
        sendframe((unsigned char *)"\x82\xFF\x02",3); // already in run mode, déjà en mode run
        }
      else
        {
        run=1;
        sendframe((unsigned char *)"\x82\x00",2); // Ok
        firstscan=1;
        }  
      }
    else
      {
      if(framin[1]==2) // Ask for stop, demande le mode stop
        {
        if(run==0)
          {
          sendframe((unsigned char *)"\x82\xFF\x03",3); // already in stop mode, déjà en mode stop
          }
         else
           {
          run=0;
          sendframe((unsigned char *)"\x82\x00",2); // Ok
           }  
        }
      else sendframe((unsigned char *)"\x82\xFF\x01",3); // unknow mode, mode inconnu
      }
    } break;
  case 4 : // I AM ALIVE, je suis vivant
    break;
  }
}

// send analog datas values, emmet les données analogiques
void senddatas(void)
{
unsigned count;
unsigned short buff[4];
if(firstscan)
  {
  firstscan=0;
  lastscantime=micros();
  }
else
  {
  if(micros()-lastscantime<mintimebetween2scansmicrosec) 
    {
    return;
    }
  lastscantime=micros();
  }
for(count=0;count<nchannels;count++)
  {
  buff[count]= analogRead(analogPin+count);
  }
sendframe((byte *)buff,2*nchannels);
}


// Main loop, boucle pirncipale
void loop(void)
{
processframein(); // read incoming chars and extract frames, lecture des caractères qui arrivent et extraction des trames
if(framereceived)
  { 
  processcmd(); // process command for received frame, traite la commande de la trame reçu
  framereceived=0;
  }
if(run)
   {
   senddatas(); // in run mode send analog values, en mode run, emmet les données alalogiques
   }
}

